/***************************************************************************
 *   Copyright (C) 2007 by Stefan Lau   *
 *   stefan.lau@gmx.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "addprofileimpl.h"
#include "browseftpimpl.h"
//#include "mainwindowimpl.h"

AddProfileImpl::AddProfileImpl(MainWindowImpl *Parent_f) {
	setupUi(this);
	SetupCaptionsAndIcons();

	Parent = Parent_f;
	NewProfile = new OProfile();

	connect(PB_Forward, SIGNAL( clicked() ), this, SLOT( NextButton() ));
	connect(PB_Back, SIGNAL( clicked() ), this, SLOT( BackButton() ));
	connect(PB_Cancel, SIGNAL( clicked() ), this, SLOT( CancelButton() ));

	connect(LW_OtherProfile, SIGNAL( currentRowChanged(int) ), this, SLOT( LW_OtherProfile_RowChanged(int) ));
	
	connect(PB_AddFile, SIGNAL( clicked() ), this, SLOT( AddFileButton() ));
	connect(PB_RemoveFile, SIGNAL( clicked() ), this, SLOT( RemoveFileButton() ));
	
	connect( &FTP, SIGNAL(commandFinished(int,bool)), SLOT(ftp_commandFinished()) );
	
	connect(CB_CheckForMDTM, SIGNAL( stateChanged(int) ), this, SLOT( CB_CheckForMDTM_Changed(int) ));
	connect(CB_CheckForSize, SIGNAL( stateChanged(int) ), this, SLOT( CB_CheckForSize_Changed(int) ));

	connect(PB_BaseDir_Local_Seek, SIGNAL( clicked() ), this, SLOT( BaseDirLocalSeekButton() ));
	connect(PB_BaseDir_Server_Seek, SIGNAL( clicked() ), this, SLOT( BaseDirServerSeekButton() ));
	
	connect(TW_Server, SIGNAL( itemClicked(QTreeWidgetItem*, int) ), this, SLOT( TW_ServerChanged(QTreeWidgetItem*, int) ));
	connect(TW_Local, SIGNAL( itemClicked(QTreeWidgetItem*, int) ), this, SLOT( TW_LocalChanged(QTreeWidgetItem*, int) ));
	
	QWidget::setTabOrder(LE_Profilename, PB_Back);
	QWidget::setTabOrder(LE_Servername, LE_Username);
	QWidget::setTabOrder(LE_Username, LE_Password);
	QWidget::setTabOrder(LE_Password, PB_Back);
	QWidget::setTabOrder(PB_Back, PB_Forward);
	QWidget::setTabOrder(PB_Forward, PB_Cancel);
	
	std::vector<OProfile*>::iterator i = Parent->Profiles.begin();
	for (;i < Parent->Profiles.end();i++)
		LW_OtherProfile->addItem((*i)->Name);

	Accepted = 0;
	PB_Back->setEnabled(0);
}

void AddProfileImpl::SetupCaptionsAndIcons() {
	this->setWindowIcon(QIcon(":/icons/oneclickftp.png"));
	//L_Image->setPixmap(QPixmap(":/icons/oneclickftp.png").scaled(41, 41));
	
	stackedWidget->setCurrentIndex(0);

	L_NewProfile->setText(tr("Add new Profile"));
	PB_Back->setText(tr("Back"));
	PB_Back->setIcon(QIcon(":/icons/back.png"));
	PB_Forward->setText(tr("Next"));
	PB_Forward->setIcon(QIcon(":/icons/forward.png"));
	PB_Cancel->setText(tr("Cancel"));
	PB_Cancel->setIcon(QIcon(":/icons/no.png"));
	
	L_Profilename->setText(tr("Enter a name for the new profile:"));
	L_Servername->setText(tr("Server:"));
	L_Username->setText(tr("Username:"));
	L_Password->setText(tr("Password:"));

	L_OtherProfiles->setText(tr("Use Settings from Profile:"));
	
	L_BaseDir_Local->setText(tr("Local Base-Direcory:"));
	L_BaseDir_Server->setText(tr("Remote Base-Directory:"));
	
	L_Local->setText(tr("Local Files"));
	L_Server->setText(tr("Remote Files"));
	PB_AddFile->setIcon(QIcon(":/icons/add.png"));
	PB_RemoveFile->setIcon(QIcon(":/icons/remove.png"));
	
	PB_AddFile->setToolTip(tr("Add a new file."));
	PB_RemoveFile->setToolTip(tr("Remove the selected files."));

	CB_CheckForMDTM->setText(tr("Process only files with newer last modified date"));
	CB_CheckForSize->setText(tr("Process only files with different sizes"));

	L_Help1->setText(tr("Welcome.\n\nThis Assistant will guide you through the creation of a new profile in OneClickFTP.\n\nThe Files you define later in this dialog can be up- or downloaded to your FTP Server with just one click."));
	L_Help2->setText(tr("At first you have to give a unique name to your profile. This name will later be listed in the menus and by clicking on this name you will be able to up- or download your files.\n\nThe name should be chosen self-explanatory, so if you are updating your Homepage with OneClickFTP it could be called \"My Homepage\"."));
	L_Help3->setText(tr("The second step is to define your server details. Here you have to input the adress, username and password of your FTP server.\n\nProxy Settings can be defined in the \"Settings\"-Dialog of the main window.\n\nBe shure to have a working internet connection before clicking next, because the server will be automatically validated."));
	L_Help4->setText(tr("Now you have to choose the so called \"base directories\" of your OneClickFTP profile.\n\nThe base directories hold all the data you want to use. This mean if all the files you want to upload lie in /home/user123, this is your local base directory. You don't need to define base directories, you can also just leave them to use your root directories-"));
	L_Help5->setText(tr("The most important thing you have to do is to define the files that will be uploaded using this profile. To add a file simply click on the small plus-sign."));
	L_Help6->setText(tr("Here you can chose if you want always all your files to be transfered (none of the boxes checked), or if you want to apply some conditions to the transfer."));
}

void AddProfileImpl::NextButton() {
	if (stackedWidget->currentIndex()==0) {
		stackedWidget->setCurrentIndex(1);
		PB_Back->setEnabled(1);
	} else
	if (stackedWidget->currentIndex()==1) {
		if (!LE_Profilename->text().isEmpty()) {
			std::vector<OProfile*>::iterator i = Parent->Profiles.begin();
			bool Exists = 0;
			for (;i < Parent->Profiles.end();i++)
				if ((*i)->Name == LE_Profilename->text())
					Exists = 1;
			if (!Exists) {
				NewProfile->Name = LE_Profilename->text();
				stackedWidget->setCurrentIndex(2);
			} else QMessageBox::information(this, tr("Error"), tr("This name already exists. Please enter another name."));
		} else QMessageBox::critical( this, tr("Error"), tr("Enter a name for the new profile."));
	} else
	if (stackedWidget->currentIndex()==2) {
		NewProfile->Server = LE_Servername->text();
		NewProfile->Username = LE_Username->text();
		NewProfile->Password =  LE_Password->text();

		connect( &FTP, SIGNAL(done(bool)), this, SLOT(CheckFTP(bool)) );
		setCursor(Qt::WaitCursor);
		if ( FTP.state() == QFtp::Connected ) FTP.close();
		FTP.connectToHost(NewProfile->Server);
		FTP.login(NewProfile->Username, NewProfile->Password);
		FTP.close();
	} else
	if (stackedWidget->currentIndex()==3) {
		if (LE_BaseDir_Local->text().isEmpty()) NewProfile->LocalBaseDir = "/";
		if (LE_BaseDir_Server->text().isEmpty()) {
			if (!Win) NewProfile->ServerBaseDir = "/";
			else NewProfile->ServerBaseDir = "C:\\";
		}
		TW_Local->headerItem()->setText(0, NewProfile->LocalBaseDir);
		TW_Server->headerItem()->setText(0, NewProfile->ServerBaseDir);
		stackedWidget->setCurrentIndex(4);
	} else
	if (stackedWidget->currentIndex()==4) {
		stackedWidget->setCurrentIndex(5);
		PB_Forward->setText(tr("Finish"));
	} else 
	if (stackedWidget->currentIndex()==5) {
		Accepted = 1;
		this->accept();
	}
}

void AddProfileImpl::BackButton() {
	stackedWidget->setCurrentIndex(stackedWidget->currentIndex()-1);
	if (stackedWidget->currentIndex()==0) PB_Back->setEnabled(0);
	PB_Forward->setText(tr("Next"));
}

void AddProfileImpl::CancelButton() {
	this->reject();
}


void AddProfileImpl::BaseDirLocalSeekButton() {
	QString BaseDir = QFileDialog::getExistingDirectory(this, tr("Chose local base-directory:"), QDir::toNativeSeparators(QDir::homePath()));
	if (!BaseDir.isEmpty()) {
		NewProfile->LocalBaseDir = BaseDir;
		LE_BaseDir_Local->setText(NewProfile->LocalBaseDir);
	}
}

void AddProfileImpl::BaseDirServerSeekButton() {
	BrowseFTPImpl *dialog = new BrowseFTPImpl(NewProfile, "", 1);
	dialog->exec();
	if (!dialog->Result.isEmpty()) {
		NewProfile->ServerBaseDir = dialog->Result;
		LE_BaseDir_Server->setText(NewProfile->ServerBaseDir);
	}
}


void AddProfileImpl::LW_OtherProfile_RowChanged(int NewRow) {
	if (NewRow >= 0) {
		LE_Servername->setText(Parent->Profiles[NewRow]->Server);
		LE_Username->setText(Parent->Profiles[NewRow]->Username);
		LE_Password->setText(Parent->Profiles[NewRow]->Password);
	}
}


void AddProfileImpl::AddFileButton() {
	QString FileLocal;
	FileLocal = QFileDialog::getSaveFileName(this, tr("Chose location for downloaded file"), NewProfile->LocalBaseDir, tr("All Files (*.*)"), 0, QFileDialog::DontConfirmOverwrite);
	if (!FileLocal.isEmpty()) {
		FileLocal.remove(0, NewProfile->LocalBaseDir.size()-1);
		FileLocal.remove(0,1);
		QString FileServer;
		BrowseFTPImpl *dialog = new BrowseFTPImpl(NewProfile, FileLocal.split("/").last());
		dialog->exec();
		if (!dialog->Result.isEmpty()) {
			dialog->Result.remove(0, NewProfile->ServerBaseDir.size()-1);
			NewProfile->Files.push_back(new OFileConnection(FileLocal, dialog->Result));
			ListFiles();
		}
	}
}

void AddProfileImpl::RemoveFileButton() {
	if ((TW_Server->selectedItems().size() == 1) && (TW_Local->selectedItems().size() == 1)) {
		QTreeWidgetItem *LocalItem = TW_Local->currentItem();
		QTreeWidgetItem *ServerItem = TW_Server->currentItem();
		
		if (!NewProfile->DeleteConnection(LocalItem, ServerItem)) QMessageBox::critical(this, tr("Error"), tr("Could not be deleted (BUG)"));;
	}
	ListFiles();
}


void AddProfileImpl::CB_CheckForMDTM_Changed(int ChangedTo) {
	if (ChangedTo == 2) {
		NewProfile->CheckForMDTM = 1;
		NewProfile->CheckForSize = 0;
		CB_CheckForSize->setCheckState(Qt::Unchecked);
	} else
	if (ChangedTo == 0)
		NewProfile->CheckForMDTM = 0;	
}

void AddProfileImpl::CB_CheckForSize_Changed(int ChangedTo) {
	if (ChangedTo == 2) {
		NewProfile->CheckForMDTM = 0;
		NewProfile->CheckForSize = 1;
		CB_CheckForMDTM->setCheckState(Qt::Unchecked);
	} else
	if (ChangedTo == 0)
		NewProfile->CheckForSize = 0;
}


void AddProfileImpl::ListFiles() {
		TW_Local->clear();
		TW_Server->clear();

		QTreeWidgetItem *Last;
		QStringList dir;
		bool Jump = 0;
		int DirCount = 0;
		QStringList temp;
		
		std::vector<OFileConnection*>::iterator it1 = NewProfile->Files.begin();
		for (; it1 < NewProfile->Files.end(); it1++) {
			temp.append((*it1)->LocalFile);
		}


		for (int i=0;i<temp.size();i++) {
			dir = temp.at(i).split("/");
			for (int j=0;j<dir.size();j++) {
				Jump = 0;
				if (j==0) Last = TW_Local->invisibleRootItem();				
				QList<QTreeWidgetItem *> ChildList = Last->takeChildren();
				for (int k=0; k<ChildList.size(); k++)
					if (ChildList.at(k)->text(0) == dir.at(j)) {
							Jump=1;
							if (j == dir.size()-1) NewProfile->Files[i]->LocalWidgetItem = ChildList.at(k);
							Last->addChildren(ChildList);
							Last=ChildList.at(k);
					}
				if (!Jump) {
					QTreeWidgetItem *Item;
					Last->addChildren(ChildList);
					
					if (j == dir.size()-1) Item = new QTreeWidgetItem(Last); else {Item = new QTreeWidgetItem(0);Last->insertChild(0, Item);DirCount++;}
					if (j == dir.size()-1) Item->setIcon(0, QIcon(":/icons/file.png")); else Item->setIcon(0, QIcon(":/icons/folder.png"));

					Item->setText(0, dir.at(j));
					Last = Item;
					if (j == dir.size()-1) NewProfile->Files[i]->LocalWidgetItem = Item;
				}		
			}
		}
		
		DirCount=0;
		temp.clear();
		std::vector<OFileConnection*>::iterator it2 = NewProfile->Files.begin();
		for (; it2 < NewProfile->Files.end(); it2++) {
			temp.append((*it2)->ServerFile);
		}
		
		for (int i=0;i<temp.size();i++) {
			dir = temp.at(i).split("/");
			for (int j=0;j<dir.size();j++) {
				Jump = 0;
				if (j==0) Last = TW_Server->invisibleRootItem();				
				QList<QTreeWidgetItem *> ChildList = Last->takeChildren();
				for (int k=0; k<ChildList.size(); k++)
					if (ChildList.at(k)->text(0) == dir.at(j)) {
							Jump = 1;
							if (j == dir.size()-1) NewProfile->Files[i]->ServerWidgetItem = ChildList.at(k);
							Last->addChildren(ChildList);
							Last=ChildList.at(k);
					}
				if (!Jump) {
					QTreeWidgetItem *Item;
					Last->addChildren(ChildList);
					
					if (j == dir.size()-1) Item = new QTreeWidgetItem(Last); else {Item = new QTreeWidgetItem(0);Last->insertChild(0, Item);DirCount++;}
					if (j == dir.size()-1) Item->setIcon(0, QIcon(":/icons/file.png")); else Item->setIcon(0, QIcon(":/icons/folder.png"));

					Item->setText(0, dir.at(j));
					Last = Item;
					if (j == dir.size()-1) NewProfile->Files[i]->ServerWidgetItem = Item;
				}		
			}
		}
}


void AddProfileImpl::TW_ServerChanged(QTreeWidgetItem *Current, int) { 
	if (Current->childCount()<1) {
		TW_Local->clearSelection();
		TW_Local->collapseAll();
		TW_Local->setSelectionMode(QAbstractItemView::MultiSelection);	
		
		std::vector<OFileConnection*> Connections = NewProfile->LookForServerWidgetItem(Current);
		std::vector<OFileConnection*>::iterator j = Connections.begin();
		for (; j < Connections.end(); j++) {
			TW_Local->setCurrentItem((*j)->LocalWidgetItem);
			(*j)->LocalWidgetItem->setSelected(1);
		}

		TW_Local->setSelectionMode(QAbstractItemView::SingleSelection);
		
		TW_Server->setSelectionMode(QAbstractItemView::SingleSelection);
		TW_Server->collapseAll();
		TW_Server->scrollToItem(Current);
		TW_Server->setCurrentItem(Current);
	}
}


void AddProfileImpl::TW_LocalChanged(QTreeWidgetItem *Current, int) {
	if (Current->childCount()<1) {
		TW_Server->clearSelection();
		TW_Server->collapseAll();
		TW_Server->setSelectionMode(QAbstractItemView::MultiSelection);
		
		std::vector<OFileConnection*> Connections = NewProfile->LookForLocalWidgetItem(Current);
		std::vector<OFileConnection*>::iterator j = Connections.begin();
		for (; j < Connections.end(); j++) {
			TW_Server->setCurrentItem((*j)->LocalWidgetItem);
			(*j)->ServerWidgetItem->setSelected(1);
		}

		TW_Server->setSelectionMode(QAbstractItemView::SingleSelection);
		
		TW_Local->setSelectionMode(QAbstractItemView::SingleSelection);
		TW_Local->collapseAll();
		TW_Local->scrollToItem(Current);
		TW_Local->setCurrentItem(Current);
	}
}


void AddProfileImpl::CheckFTP(bool Error) {
	setCursor(Qt::ArrowCursor);
	if (!Error) {
		stackedWidget->setCurrentIndex(3);	
	}
	else QMessageBox::critical( this, tr("Could not validate the server"), tr("The application couldn't connect to the server￼\n")+FTP.errorString() );
	disconnect( &FTP, SIGNAL(done(bool)), this, SLOT(CheckFTP(bool)) );
	if ( FTP.state() == QFtp::Connected ) FTP.close();
}

void AddProfileImpl::ftp_commandFinished()
{
    delete FTP.currentDevice(); 
}
