/***************************************************************************
 *   Copyright (C) 2007 by Stefan Lau   *
 *   stefan.lau@gmx.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
 #include "ftpwindowimpl.h"

FTPWindowImpl::FTPWindowImpl(QString Dir, QString Profile, QString ProxyIP, QString ProxyPort)
{

	setupUi(this);
	this->setAttribute(Qt::WA_DeleteOnClose);
	
	if (ProxyIP != 0) {
		FTP.setProxy(ProxyIP, ProxyPort.toInt());
	}
	
	//From Forum
    	connect( &FTP, SIGNAL(commandFinished(int,bool)), SLOT(ftp_commandFinished(int,bool)) );
    	connect( &FTP, SIGNAL(done(bool)), SLOT(ftp_done(bool)) );

	connect(PB_Cancel, SIGNAL( clicked() ), this, SLOT( PB_Cancel_clicked() ));
	connect(&FTP, SIGNAL( dataTransferProgress(qint64,qint64) ), this, SLOT( Progress(qint64, qint64) ));
	
	connect(&ProgressTimer, SIGNAL( timeout() ), this, SLOT( ProgressTimerEvent() ));
	
	this->setWindowTitle(Dir+Profile);
	SetupCaptionsAndIcons();
}

void FTPWindowImpl::SetupCaptionsAndIcons() {
	this->setWindowIcon(QIcon(":/icons/oneclickftp.png"));
	
	L_Total->setText(tr("Total"));
	PB_Cancel->setText(tr("Cancel"));
	PB_Cancel->setIcon(QIcon(":/icons/no.png"));
}

bool FTPWindowImpl::Upload(OProfile *ProfiletoUpload, std::vector<OFileConnection*> SelectedFiles) {
	Everything = 0;
	for (std::vector<OFileConnection*>::iterator i = SelectedFiles.begin(); i < SelectedFiles.end(); i++) {
		if (!(*i)->ExistsLocal) {
			QMessageBox::information(this, tr("Information"), "\"" + (*i)->LocalFile + "\" does not exist localy and therefore will not be uploaded.");
			SelectedFiles.erase(i);
		} else Everything = Everything +(*i)->SizeLocal;
	}

	LastFile = NULL;

	DoneTotalBefore = 0;

	Downloading = 0;
	CurrentFile = -1;
	Profile = ProfiletoUpload;
	FilesToProcess = SelectedFiles;
	Aborted = 0;


	L_File->setText(tr("Connecting."));
	PBar_File->setRange(0,-1);
	PBar_Total->setRange(0,-1);

	FTP.connectToHost(Profile->Server);
	FTP.login(Profile->Username, Profile->Password);
	
	FTP.cd(Profile->ServerBaseDir);

	PBar_Total->setRange(0,Everything);
	L_File->setText(FilesToProcess[0]->LocalFile.split("/").last()+tr(" - File ")+"1"+tr(" of ")+QString::number(FilesToProcess.size()));
	UploadFile();

	return 1;	
}

bool FTPWindowImpl::Download(OProfile *ProfiletoDownload, std::vector<OFileConnection*> SelectedFiles) {
	Everything = 0;
	for (std::vector<OFileConnection*>::iterator i = SelectedFiles.begin(); i < SelectedFiles.end(); i++) {
		if (!(*i)->ExistsServer) {
			QMessageBox::information(this, tr("Information"), "\"" + (*i)->ServerFile + "\" does not exist and therefore will not be downloaded.");
			SelectedFiles.erase(i);
		} else Everything = Everything +(*i)->SizeServer;
	} 

	LastFile = NULL;

	DoneTotalBefore = 0;

	Downloading = 1;
	CurrentFile = -1;
	Profile = ProfiletoDownload;
	FilesToProcess = SelectedFiles;
	Aborted = 0;

	L_File->setText(tr("Connecting."));
	PBar_File->setRange(0,-1);
	PBar_Total->setRange(0,-1);

	FTP.connectToHost(Profile->Server);
	FTP.login(Profile->Username, Profile->Password);

	FTP.cd(Profile->ServerBaseDir);

	PBar_Total->setRange(0,Everything);
	L_File->setText(FilesToProcess[0]->LocalFile.split("/").last()+tr(" - File ")+"1"+tr(" of ")+QString::number(FilesToProcess.size()));
	DownloadFile();

	return 1;		
}

bool FTPWindowImpl::DownloadFile() {
	if (CurrentFile >= 0) DoneTotalBefore += FilesToProcess[CurrentFile]->SizeServer;
	CurrentFile++;

	/*if (LastFile != NULL) {
		LastFile->close();
		delete LastFile;
	}*/

	if (CurrentFile == (signed)FilesToProcess.size()) return 0;

	L_File->setText(FilesToProcess[CurrentFile]->ServerFile.split("/").last()+tr(" - File ")+QString::number(CurrentFile+1)+tr(" of ")+QString::number(FilesToProcess.size()));
	if (ProgressTimer.isActive()) ProgressTimer.stop();
	ProgressTimer.setInterval(10);
	ProgressTimer.start();
	DoneBefore = 0;
	TimerSinceLastCheck = 0;
	Average = 0;

	QFile *File = new QFile();
	LastFile = File;
	File->setFileName(Profile->LocalBaseDir + FilesToProcess[CurrentFile]->LocalFile + "~oneclickftp");
	if (!File->open(QIODevice::WriteOnly)) {
		QMessageBox::critical(this, tr("Error"), tr("Could not write to ") + Profile->LocalBaseDir + FilesToProcess[CurrentFile]->LocalFile + "~oneclickftp");
		return 0;
	}

	QStringList dir;
	dir = FilesToProcess[CurrentFile]->ServerFile.split("/");
	for (int u=0; u<dir.size()-1;u++) FTP.cd(dir.at(u));

	if (Profile->CheckForSize) {
		if (FilesToProcess[CurrentFile]->SizeLocal != FilesToProcess[CurrentFile]->SizeServer) {
			FTP.get(dir.last(), File);
		}
	} else
	if (Profile->CheckForMDTM) {
		QDateTime MDTML = QDateTime::fromString(FilesToProcess[CurrentFile]->LastModifiedLocal, "yyyyMMddhhmmss");
		QDateTime MDTMS = QDateTime::fromString(FilesToProcess[CurrentFile]->LastModifiedServer, "yyyyMMddhhmmss");
		MDTMS = MDTMS.addSecs(-3600 * Profile->TimeDifference);
		//QMessageBox::information(this, "MDTMS", MDTML.toString() + " - " + MDTMS.toString());
		if (MDTMS > MDTML) {
			FTP.get(dir.last(), File);
		}
	} else
	if (!Profile->CheckForSize && !Profile->CheckForMDTM) {
		FTP.get(dir.last(), File);
	}
	
	for (int u=0; u<dir.size()-1;u++) FTP.cd("..");

	return 1;
}

bool FTPWindowImpl::UploadFile() {
	if (CurrentFile >= 0) DoneTotalBefore += FilesToProcess[CurrentFile]->SizeLocal;
	CurrentFile++;

	/*if (LastFile != NULL) {
		LastFile->close();
		delete LastFile;
	}*/

	if (CurrentFile == (signed)FilesToProcess.size()) return 0;

	L_File->setText(FilesToProcess[CurrentFile]->LocalFile.split("/").last()+tr(" - File ")+QString::number(CurrentFile + 1)+tr(" of ")+QString::number(FilesToProcess.size()));
	if (ProgressTimer.isActive()) ProgressTimer.stop();
	ProgressTimer.setInterval(10);
	ProgressTimer.start();
	DoneBefore = 0;
	TimerSinceLastCheck = 0;
	Average = 0;

	QFile *File = new QFile;
	File->setFileName(Profile->LocalBaseDir + FilesToProcess[CurrentFile]->LocalFile);
	if (!File->open(QIODevice::ReadOnly)) {
		QMessageBox::critical(this, tr("Error"), tr("Could not read from file ") + Profile->LocalBaseDir + FilesToProcess[CurrentFile]->LocalFile + "~oneclickftp");
		return 0;
	}


	QStringList dir;
	dir = FilesToProcess[CurrentFile]->ServerFile.split("/");

	for (int u=0; u<dir.size()-1;u++) FTP.cd(dir.at(u));
	
	if (Profile->CheckForSize) {
		if (FilesToProcess[CurrentFile]->SizeLocal != FilesToProcess[CurrentFile]->SizeServer) {
			FTP.put(File, dir.last() + "~oneclickftp");
		}
	} else
	if (Profile->CheckForMDTM) {
		QDateTime MDTML = QDateTime::fromString(FilesToProcess[CurrentFile]->LastModifiedLocal, "yyyyMMddhhmmss");
		QDateTime MDTMS = QDateTime::fromString(FilesToProcess[CurrentFile]->LastModifiedServer, "yyyyMMddhhmmss");
		MDTMS = MDTMS.addSecs(-3600 * Profile->TimeDifference);
		//QMessageBox::information(this, "MDTMS", MDTML.toString() + " - " + MDTMS.toString());
		if (MDTMS < MDTML) {
			FTP.put(File, dir.last() + "~oneclickftp");
		}
	} else
	if (!Profile->CheckForSize && !Profile->CheckForMDTM) {
		FTP.put(File, dir.last() + "~oneclickftp");
	}

	File->close();

	return 1;
}

void FTPWindowImpl::ftp_commandFinished(int, bool Error)
{
	if (!Aborted) {
			if (FTP.currentCommand() == 9 && !Error) {
			QStringList dir = FilesToProcess[CurrentFile]->ServerFile.split("/");
			FTP.rename(dir.last() + "~oneclickftp", dir.last());
			for (int u=0; u<dir.size()-1;u++) FTP.cd("..");
			
			QFile *File = new QFile();
			File->setFileName(Profile->LocalBaseDir + FilesToProcess[CurrentFile]->LocalFile);
			File->open(QIODevice::ReadWrite);
			File->resize(File->size()+1);
			File->resize(File->size()-1);
			File->close();

			if (CurrentFile + 1 == (signed)FilesToProcess.size()) {
				FTP.close();
			}
			UploadFile();
		}

		if (FTP.currentCommand() == 8 && !Error) {
			QDir Directory(Profile->LocalBaseDir);
			if (Directory.exists(FilesToProcess[CurrentFile]->LocalFile)) Directory.remove(FilesToProcess[CurrentFile]->LocalFile);
			Directory.rename(FilesToProcess[CurrentFile]->LocalFile + "~oneclickftp", FilesToProcess[CurrentFile]->LocalFile);

			if (CurrentFile + 1 == (signed)FilesToProcess.size()) {
				FTP.close();
			}
			DownloadFile();
		}
	}
	delete FTP.currentDevice();
}

void FTPWindowImpl::Progress(qint64 done, qint64 total) {
	if (TimerSinceLastCheck != 0 && TimerTotal != 0) {
		double Velo = ((done-DoneBefore) / TimerSinceLastCheck) * 100;

		L_Velocity->setText(SizeToString(done, total) + tr("Velocity: ")+QString::number(Velo/1024, 'f', 2)+tr(" KB/s") + ";");
	}

	PBar_File->setRange(0,total);
	PBar_File->setValue(done);

	PBar_Total->setRange(0, Everything);
	PBar_Total->setValue(DoneTotalBefore + done);

	DoneBefore = done;
	TimerSinceLastCheck = 0;
}

void FTPWindowImpl::ftp_done( bool error )
{
	if ( FTP.state() == QFtp::Connected ) FTP.close();
	if ( error ) {
		QMessageBox::critical( this, tr("FTP Error"), FTP.errorString() );
	}
	this->close();
}

void FTPWindowImpl::ProgressTimerEvent() {
	TimerSinceLastCheck++;
	TimerTotal++;
}

void FTPWindowImpl::PB_Cancel_clicked() {
	this->setEnabled(0);
	L_Velocity->setText(tr("Cancelling"));

	Aborted = 1;
	FTP.clearPendingCommands();
	FTP.abort();

	if (Downloading) {
		QDir Directory(Profile->LocalBaseDir);
		Directory.remove(FilesToProcess[CurrentFile]->LocalFile+"~oneclickftp");
	} else
		FTP.remove(FilesToProcess[CurrentFile]->ServerFile.split("/").last()+"~oneclickftp");
}

QString FTPWindowImpl::SizeToString(int Done, int Total) {
	QString DoneE;
	QString TotalE;
	double DoneD = Done;
	double TotalD = Total;
	if (DoneD >= 1024) {
		DoneD = DoneD / 1024;
		DoneE = "KB";
	}
	if (DoneD >= 1024) {
		DoneD = DoneD / 1024;
		DoneE = "MB";
	}
	if (DoneD >= 1024) {
		DoneD = DoneD / 1024;
		DoneE = "GB";
	}
	if (DoneD >= 1024) {
		DoneD = DoneD / 1024;
		DoneE = "TB";
	}
	if (TotalD >= 1024) {
		TotalD = TotalD / 1024;
		TotalE = "KB";
	}
	if (TotalD >= 1024) {
		TotalD = TotalD / 1024;
		TotalE = "MB";
	}
	if (TotalD >= 1024) {
		TotalD = TotalD / 1024;
		TotalE = "GB";
	}
	if (TotalD >= 1024) {
		TotalD = TotalD / 1024;
		TotalE = "TB";
	}

	return tr("Done: ")+QString::number(DoneD, 'f', 1)+" " + DoneE + tr(" of ") + QString::number(TotalD, 'f', 1) + " " + TotalE + "; ";
	}