/***************************************************************************
 *   Copyright (C) 2007 by Stefan Lau   *
 *   stefan.lau@gmx.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "mainwindowimpl.h"
#include "ftpwindowimpl.h"
#include "browseftpimpl.h"
#include "addprofileimpl.h"
#include "settingsimpl.h"
#include "exportprofilesimpl.h"
#include "updatefilestatusimpl.h"
#include "fileinfodialogimpl.h"
#include <QtCrypto>
#include <QBuffer>

MainWindowImpl::MainWindowImpl( QWidget * parent, Qt::WFlags f) 
	: QMainWindow(parent, f)
{
	setupUi(this);
	Changed = 0;

	this->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed); 
	
	Settings = new QSettings(QDir::homePath()+"/.oneclickftp/oneclickftp.conf", QSettings::IniFormat, this);
	Home.setPath(QDir::toNativeSeparators(QDir::homePath()));

	OldProfile = new OProfile;
	
	connect(&TrayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, SLOT(ShowHide(QSystemTrayIcon::ActivationReason)));
	
	connect(TW_Server, SIGNAL( itemClicked(QTreeWidgetItem*, int) ), this, SLOT( TW_ServerChanged(QTreeWidgetItem*, int) ));
	connect(TW_Local, SIGNAL( itemClicked(QTreeWidgetItem*, int) ), this, SLOT( TW_LocalChanged(QTreeWidgetItem*, int) ));

	connect(TW_Server, SIGNAL( itemDoubleClicked(QTreeWidgetItem*, int) ), this, SLOT( TW_Server_itemDoubleClicked(QTreeWidgetItem*, int) ));
	connect(TW_Local, SIGNAL( itemDoubleClicked(QTreeWidgetItem*, int) ), this, SLOT( TW_Local_itemDoubleClicked(QTreeWidgetItem*, int) ));

	connect(qobject_cast<QWidget*>(TW_Server), SIGNAL( customContextMenuRequested(const QPoint &) ), this, SLOT( TW_Server_rightClick(const QPoint &) ));
	connect(qobject_cast<QWidget*>(TW_Local), SIGNAL( customContextMenuRequested(const QPoint &) ), this, SLOT( TW_Local_rightClick(const QPoint &) ));
	
	connect(PB_Upload, SIGNAL( clicked() ), this, SLOT( StartUploadButton() ));
	connect(PB_Download, SIGNAL( clicked() ), this, SLOT( StartDownloadButton() ));
	
	connect(PB_AddProfile, SIGNAL( clicked() ), this, SLOT( AddProfileButton() ));
	connect(PB_RemoveProfile, SIGNAL( clicked() ), this, SLOT( RemoveProfileButton() ));
	connect(PB_RenameProfile, SIGNAL( clicked() ), this, SLOT( RenameProfileButton() ));
	
	connect(PB_AddFile, SIGNAL( clicked() ), this, SLOT( AddFileButton() ));
	connect(PB_RemoveFile, SIGNAL( clicked() ), this, SLOT( RemoveFileButton() ));
	connect(PB_UpdateFileStatus, SIGNAL( clicked() ), this, SLOT( UpdateFileStatusButton() ));
	
	connect(PB_BaseDir_Local_Seek, SIGNAL( clicked() ), this, SLOT( BaseDirLocalSeekButton() ));
	connect(PB_BaseDir_Server_Seek, SIGNAL( clicked() ), this, SLOT( BaseDirServerSeekButton() ));
	
	connect(LW_Profiles, SIGNAL( currentRowChanged(int) ), this, SLOT( LWProfilesRowChanged(int) ));
	connect(LE_Servername, SIGNAL( textEdited(QString) ), this, SLOT( LE_Changed(QString) ));
	connect(LE_Username, SIGNAL( textEdited(QString) ), this, SLOT( LE_Changed(QString) ));
	connect(LE_Password, SIGNAL( textEdited(QString) ), this, SLOT( LE_Changed(QString) ));
	connect(CB_CheckForMDTM, SIGNAL( stateChanged(int) ), this, SLOT( CB_CheckForMDTM_Changed(int) ));
	connect(CB_CheckForSize, SIGNAL( stateChanged(int) ), this, SLOT( CB_CheckForSize_Changed(int) ));
}

bool MainWindowImpl::CheckForPassword(bool Startup) {
	QCA::Initializer Init;
	QCA::Hash Hash("md5");
	QString Setting;
	QByteArray Key;
	if (Startup)  {
		Setting = "EnterPasswordOnStartup";
		Key.append("Hell%No");
	} else {
		Setting = "EnterPasswordOnTransfer";
		Key.append("Not!At!All");
	}
	
	if (!(Settings->value(Setting, Hash.hash(Key).toByteArray()) == Hash.hash(Key).toByteArray())) {
		bool Ok = 1;
		while (Ok) {
			QString Password = QInputDialog::getText(this, tr("Password"), tr("Please enter the Password."), QLineEdit::Password, "", &Ok);
			QByteArray Array;
			Array.append(Password);
			if (!Ok) return 0;
			if (Ok && (Settings->value("Password") == Hash.hash(Array).toByteArray())) return 1;
				else QMessageBox::information(this, tr("Error"), tr("Wrong Password entered. Please try again."));
		}
	} else return 1;
return 0;
}

void MainWindowImpl::ShowHide(QSystemTrayIcon::ActivationReason reason) {
	if ((reason == 3) && !(this->isVisible())) {
		this->show();
	}
	else if ((reason == 3) && this->isVisible()) {
		this->hide();
		this->TrayIcon.showMessage(tr("OneClickFTP"), tr("OneClickFTP is minimized and now located in the system-tray"), QSystemTrayIcon::Information, 2000);
	}
}

void MainWindowImpl::TW_ServerChanged(QTreeWidgetItem *Current, int) {
	if (Current->childCount()<1) {
		TW_Local->clearSelection();
		TW_Local->collapseAll();
		TW_Local->setSelectionMode(QAbstractItemView::MultiSelection);
		int i = LW_Profiles->currentRow();		
		
		std::vector<OFileConnection*> Connections = Profiles[i]->LookForServerWidgetItem(Current);
		for (std::vector<OFileConnection*>::iterator j = Connections.begin(); j < Connections.end(); j++) {
			TW_Local->setCurrentItem((*j)->LocalWidgetItem);
			(*j)->LocalWidgetItem->setSelected(1);
		}

		TW_Local->setSelectionMode(QAbstractItemView::SingleSelection);
		
		TW_Server->setSelectionMode(QAbstractItemView::SingleSelection);
		TW_Server->collapseAll();
		TW_Server->scrollToItem(Current);
		TW_Server->setCurrentItem(Current);
	}
}

void MainWindowImpl::TW_LocalChanged(QTreeWidgetItem *Current, int) {
	if (Current->childCount()<1) {
		TW_Server->clearSelection();
		TW_Server->collapseAll();
		TW_Server->setSelectionMode(QAbstractItemView::MultiSelection);
		int i = LW_Profiles->currentRow();
		
		std::vector<OFileConnection*> Connections = Profiles[i]->LookForLocalWidgetItem(Current);
		for (std::vector<OFileConnection*>::iterator j = Connections.begin(); j < Connections.end(); j++) {
			TW_Server->setCurrentItem((*j)->ServerWidgetItem);
			(*j)->ServerWidgetItem->setSelected(1);
		}

		TW_Server->setSelectionMode(QAbstractItemView::SingleSelection);

		TW_Local->setSelectionMode(QAbstractItemView::SingleSelection);
		TW_Local->collapseAll();
		TW_Local->scrollToItem(Current);
		TW_Local->setCurrentItem(Current);
	}
}


void MainWindowImpl::TW_Server_itemDoubleClicked(QTreeWidgetItem *Current, int Row) {
	TW_ServerChanged(Current, Row);
	int i = LW_Profiles->currentRow();
	std::vector<OFileConnection*> Connections = Profiles[i]->LookForServerWidgetItem(Current);

	if (UpdateFileStatusButton()) {
		FTPWindowImpl *dialog;
		if (Settings->value("Proxy/UseProxy", 0).toBool())
			dialog = new FTPWindowImpl(tr("Downloading: "), Profiles[i]->Name, Settings->value("Proxy/ProxyIP", "127.0.0.1").toString(), Settings->value("Proxy/ProxyPort", "80").toString());
		else
			dialog = new FTPWindowImpl(tr("Downloading Profile: "), Profiles[i]->Name);

		dialog->show();
		dialog->Download(Profiles[i], Connections);
	}
}

void MainWindowImpl::TW_Local_itemDoubleClicked(QTreeWidgetItem *Current, int Row) {
	TW_LocalChanged(Current, Row);
	int i = LW_Profiles->currentRow();
	std::vector<OFileConnection*> Connections = Profiles[i]->LookForLocalWidgetItem(Current);

	if (UpdateFileStatusButton()) {
		FTPWindowImpl *dialog;
		if (Settings->value("Proxy/UseProxy", 0).toBool())
			dialog = new FTPWindowImpl(tr("Uploading: "), Profiles[i]->Name, Settings->value("Proxy/ProxyIP", "127.0.0.1").toString(), Settings->value("Proxy/ProxyPort", "80").toString());
		else
			dialog = new FTPWindowImpl(tr("Uploading Profile: "), Profiles[i]->Name);

		dialog->show();
		dialog->Upload(Profiles[i], Connections);
	}
}

void MainWindowImpl::TW_Server_rightClick(const QPoint & iPoint) {
	QTreeWidgetItem * ClickedItem = 0 ;
	ClickedItem = TW_Server->itemAt(iPoint) ;

	int i = LW_Profiles->currentRow();

	if (ClickedItem != 0 && Profiles[i]->LookForServerWidgetItem(ClickedItem).size() > 0) {
		TW_ServerChanged(ClickedItem, 0);	

		QMenu menu(TW_Server);
		menu.addAction(QIcon(":/icons/rename.png"), tr("Rename"), this, SLOT(RenameServerFile()));
		menu.addAction(QIcon(":/icons/remove.png"), tr("Delete"), this, SLOT(RemoveFileButton()));
		menu.addAction(QIcon(":/icons/info.png"), tr("File Information"), this, SLOT(ShowInfoBox()));

		menu.exec(TW_Server->mapToGlobal(iPoint));
	}

}

void MainWindowImpl::TW_Local_rightClick(const QPoint & iPoint) {
	QTreeWidgetItem * ClickedItem = 0 ;
	ClickedItem = TW_Local->itemAt(iPoint) ;

	int i = LW_Profiles->currentRow();

	if (ClickedItem != 0 && Profiles[i]->LookForLocalWidgetItem(ClickedItem).size() > 0) {
		TW_LocalChanged(ClickedItem, 0);

		QMenu menu(TW_Local);
		menu.addAction(QIcon(":/icons/rename.png"), tr("Rename"), this, SLOT(RenameLocalFile()));
		menu.addAction(QIcon(":/icons/remove.png"), tr("Delete"), this, SLOT(RemoveFileButton()));
		menu.addAction(QIcon(":/icons/info.png"), tr("File Information"), this, SLOT(ShowInfoBox()));

		menu.exec(TW_Local->mapToGlobal(iPoint));
	}
}

void MainWindowImpl::ShowProfiles(int selected) {
	LW_Profiles->clear();
	for (std::vector<OProfile*>::iterator i = Profiles.begin(); i < Profiles.end(); i++) {
		LW_Profiles->addItem((*i)->Name);
	}
	if ((signed)Profiles.size() > selected) {
		LW_Profiles->setCurrentRow(selected);
		ShowProfileDetails(selected);
	}
}

void MainWindowImpl::ShowProfileDetails(int selected) {
	if (Profiles.size() != 0) {
		TW_Local->clear();
		TW_Server->clear();

		TW_Local->headerItem()->setText(0, Profiles[selected]->LocalBaseDir);
		TW_Server->headerItem()->setText(0, Profiles[selected]->ServerBaseDir);
		
		CB_CheckForMDTM->setChecked(Profiles[selected]->CheckForMDTM);
		CB_CheckForSize->setChecked(Profiles[selected]->CheckForSize);

		QTreeWidgetItem *Last;
		QStringList dir;
		bool Jump = 0;
		int DirCount = 0;
		QStringList temp;
		
		for (std::vector<OFileConnection*>::iterator i = Profiles[selected]->Files.begin(); i < Profiles[selected]->Files.end(); i++)
			temp.append((*i)->LocalFile);

		for (int i=0;i<temp.size();i++) {
			dir = temp.at(i).split("/");
			for (int j=0;j<dir.size();j++) {
				Jump = 0;
				if (j==0) Last = TW_Local->invisibleRootItem();				
				QList<QTreeWidgetItem *> ChildList = Last->takeChildren();
				for (int k=0; k<ChildList.size(); k++)
					if (ChildList.at(k)->text(0) == dir.at(j)) {
							Jump=1;
							if (j == dir.size()-1) Profiles[selected]->Files[i]->LocalWidgetItem = ChildList.at(k);
							Last->addChildren(ChildList);
							Last=ChildList.at(k);
					}
				if (!Jump) {
					QTreeWidgetItem *Item;
					Last->addChildren(ChildList);
					
					if (j == dir.size()-1) Item = new QTreeWidgetItem(Last); else {Item = new QTreeWidgetItem(0);Last->insertChild(0, Item);DirCount++;}
					if (j == dir.size()-1) {

						/*Icon Auswahl*/
						if (Profiles[selected]->Files[i]->ExistsLocal) {
							Item->setIcon(0, QIcon(":/icons/file.png"));
							if (Profiles[selected]->Files[i]->ExistsServer) {
								QDateTime MDTML = QDateTime::fromString(Profiles[selected]->Files[i]->LastModifiedLocal, "yyyyMMddhhmmss");
								QDateTime MDTMS = QDateTime::fromString(Profiles[selected]->Files[i]->LastModifiedServer, "yyyyMMddhhmmss");
								MDTMS = MDTMS.addSecs(-3600 * Profiles[selected]->TimeDifference);
								if (MDTML > MDTMS) Item->setIcon(0, QIcon(":/icons/file_new.png")); else
								if (MDTML < MDTMS) Item->setIcon(0, QIcon(":/icons/file_old.png"));
							}
						}
						else Item->setIcon(0, QIcon(":/icons/file_unexistant.png"));
						if (Profiles[selected]->Files[i]->New) Item->setIcon(0, QIcon(":/icons/file.png"));
					} else Item->setIcon(0, QIcon(":/icons/folder.png"));

					Item->setText(0, dir.at(j));
					Last = Item;
					if (j == dir.size()-1) Profiles[selected]->Files[i]->LocalWidgetItem = Item;
				}		
			}
		}
		
		DirCount=0;
		temp.clear();
		for (std::vector<OFileConnection*>::iterator i = Profiles[selected]->Files.begin(); i < Profiles[selected]->Files.end(); i++)
			temp.append((*i)->ServerFile);
		
		for (int i=0;i<temp.size();i++) {
			dir = temp.at(i).split("/");
			for (int j=0;j<dir.size();j++) {
				Jump = 0;
				if (j==0) Last = TW_Server->invisibleRootItem();				
				QList<QTreeWidgetItem *> ChildList = Last->takeChildren();
				for (int k=0; k<ChildList.size(); k++)
					if (ChildList.at(k)->text(0) == dir.at(j)) {
							Jump = 1;
							if (j == dir.size()-1) Profiles[selected]->Files[i]->ServerWidgetItem = ChildList.at(k);
							Last->addChildren(ChildList);
							Last=ChildList.at(k);
					}
				if (!Jump) {
					QTreeWidgetItem *Item;
					Last->addChildren(ChildList);
					
					if (j == dir.size()-1) Item = new QTreeWidgetItem(Last); else {Item = new QTreeWidgetItem(0);Last->insertChild(0, Item);DirCount++;}
					if (j == dir.size()-1) {
						
						/*Icon Auswahl*/
						if (Profiles[selected]->Files[i]->ExistsServer) {
							Item->setIcon(0, QIcon(":/icons/file.png"));
							if (Profiles[selected]->Files[i]->ExistsLocal) {
								QDateTime MDTML = QDateTime::fromString(Profiles[selected]->Files[i]->LastModifiedLocal, "yyyyMMddhhmmss");
								QDateTime MDTMS = QDateTime::fromString(Profiles[selected]->Files[i]->LastModifiedServer, "yyyyMMddhhmmss");
								MDTMS = MDTMS.addSecs(-3600 * Profiles[selected]->TimeDifference);
								if (MDTML > MDTMS) Item->setIcon(0, QIcon(":/icons/file_old.png")); else
								if (MDTML < MDTMS) Item->setIcon(0, QIcon(":/icons/file_new.png"));
							}
							if (Profiles[selected]->Files[i]->New) Item->setIcon(0, QIcon(":/icons/file.png"));
						}
						else Item->setIcon(0, QIcon(":/icons/file_unexistant.png"));


					} else Item->setIcon(0, QIcon(":/icons/folder.png"));

					Item->setText(0, dir.at(j));
					Last = Item;
					if (j == dir.size()-1) Profiles[selected]->Files[i]->ServerWidgetItem = Item;
				}		
			}
		}
	
		TW_Server->sortItems(0, Qt::AscendingOrder);
		TW_Local->sortItems(0, Qt::AscendingOrder);

		LE_Servername->setText(Profiles[selected]->Server);
		LE_Username->setText(Profiles[selected]->Username);
		LE_Password->setText(Profiles[selected]->Password);
		
		LE_BaseDir_Local->setText(Profiles[selected]->LocalBaseDir);
		LE_BaseDir_Server->setText(Profiles[selected]->ServerBaseDir);

		OldProfile->Server = Profiles[selected]->Server;
		OldProfile->Username = Profiles[selected]->Username;
		OldProfile->Password = Profiles[selected]->Password;
		OldProfile->LocalBaseDir = Profiles[selected]->LocalBaseDir;
		OldProfile->ServerBaseDir = Profiles[selected]->ServerBaseDir;
		OldProfile->CheckForMDTM  = Profiles[selected]->CheckForMDTM;
		OldProfile->CheckForSize =  Profiles[selected]->CheckForSize;
	}
}

void MainWindowImpl::SetupCaptionsAndIcons() {
	//TrayIcon
	TrayIcon.setIcon(QIcon(":/icons/oneclickftp.png"));
	TrayIcon.setVisible(1);
	//Left
	L_Profiles->setText(tr("Available profiles"));
	PB_AddProfile->setIcon(QIcon(":/icons/add.png"));
	PB_RemoveProfile->setIcon(QIcon(":/icons/remove.png"));
	PB_RenameProfile->setIcon(QIcon(":/icons/rename.png"));
	//PB_ExportProfile->setIcon(QIcon(":/icons/export.png"));
	//PB_ImportProfile->setIcon(QIcon(":/icons/import.png"));
	//Tab Widget - Server
	TW_Settings->setTabText(0, tr("Server"));
	L_Servername->setText(tr("Server"));
	L_Username->setText(tr("Username"));
	L_Password->setText(tr("Password"));
	//Tab Widget - Veichnisse
	TW_Settings->setTabText(1, tr("Directories"));
	L_BaseDir_Local->setText(tr("Local Base-Direcory"));
	L_BaseDir_Server->setText(tr("Remote Base-Direcory"));
	//Tab Widget - Sonstiges
	TW_Settings->setTabText(2, tr("Other Settings"));
	CB_CheckForMDTM->setText(tr("Process only files with newer last modified date"));
	CB_CheckForSize->setText(tr("Process only files with different sizes"));
	//Dateieinstellungen
	GB_File->setTitle(tr("Files"));
	L_FilesLocal->setText(tr("Local Files"));
	L_FilesServer->setText(tr("Remote Files"));
	PB_AddFile->setIcon(QIcon(":/icons/add.png"));
	PB_RemoveFile->setIcon(QIcon(":/icons/remove.png"));
	PB_UpdateFileStatus->setIcon(QIcon(":/icons/reload.png"));
	//Down/Up Buttons
	PB_Download->setText(tr("Download"));
	PB_Download->setIcon(QIcon(":/icons/download.png"));
	PB_Upload->setText(tr("Upload"));
	PB_Upload->setIcon(QIcon(":/icons/upload.png"));
	
	PB_AddProfile->setToolTip(tr("Add new profile"));
	PB_RemoveProfile->setToolTip(tr("Remove selected profile"));
	PB_RenameProfile->setToolTip(tr("Rename selected profile"));
	PB_AddFile->setToolTip(tr("Add a new file to the selected profile"));
	PB_RemoveFile->setToolTip(tr("Remove the selected file"));
	PB_Download->setToolTip(tr("Download selected Profile"));
	PB_Upload->setToolTip(tr("Upload selected profile"));
	
	Translator = tr("Translated By: <Name of the Translator>");
}

void MainWindowImpl::SetupMenus() {
	QMenu *download = new QMenu(tr("Download"));
	download->setIcon(QIcon(":/icons/download.png"));
	for (std::vector<OProfile*>::iterator i = Profiles.begin(); i < Profiles.end(); i++) download->addAction((*i)->Name);
	connect(download, SIGNAL( triggered(QAction*) ), this, SLOT( StartDownload(QAction*) ));
	
	QMenu *upload = new QMenu(tr("Upload"));
	upload->setIcon(QIcon(":/icons/upload.png"));
	for (std::vector<OProfile*>::iterator i = Profiles.begin(); i < Profiles.end(); i++) upload->addAction((*i)->Name);
	connect(upload, SIGNAL( triggered(QAction*) ), this, SLOT( StartUpload(QAction*) ));
		
	QMenu *traymenu = new QMenu(tr("OneClickFTP"));
	traymenu->addAction(QIcon(":/icons/show.png"), tr("Show main-window"), this, SLOT( show() ));
	traymenu->addMenu(download);
	traymenu->addMenu(upload);
	traymenu->addAction(QIcon(":/icons/quit.png"), tr("Quit"), this, SLOT( ExitApp() ));
	TrayIcon.setContextMenu(traymenu);
	
	this->menuBar()->clear();
	
	QMenu *file = new QMenu(tr("File"));
	file->addAction(QIcon(":/icons/hide.png"), tr("Hide main-window"), this, SLOT(hide()));
	file->addMenu(download);
	file->addMenu(upload);
	file->addAction(QIcon(":/icons/general.png"), tr("Settings"), this, SLOT(SettingsButton()));
	file->addAction(QIcon(":/icons/export.png"), tr("Export profile(s)"), this, SLOT(ExportProfilesButton()));
	file->addAction(QIcon(":/icons/import.png"), tr("Import profile(s)"), this, SLOT(ImportProfilesButton()));
	file->addAction(QIcon(":/icons/quit.png"), tr("Quit"), this, SLOT( ExitApp() ));
	this->menuBar()->addMenu(file);
	
	QMenu *help = new QMenu(tr("Help"));
	//help->addAction(tr("Hilfe"), this, SLOT(Help()));
	help->addAction(QIcon(":/icons/info.png"), tr("About"), this, SLOT(AboutApp()));
	help->addAction(QIcon(":/icons/info.png"), tr("About Qt"), qApp, SLOT( aboutQt() ));
	this->menuBar()->addMenu(help);
}

void MainWindowImpl::StartUpload(QAction *Profile) {
	if (CheckForPassword(0)) {
		int i = 0;
		while (Profiles[i]->Name != Profile->text()) i++;
		LW_Profiles->setCurrentRow(i);
		UpdateFileStatusButton();
		FTPWindowImpl *dialog;
		if (Settings->value("Proxy/UseProxy", 0).toBool()) dialog = new FTPWindowImpl(tr("Uploading: "), Profile->text(), Settings->value("Proxy/ProxyIP", "127.0.0.1").toString(), Settings->value("Proxy/ProxyPort", "80").toString()); else
			dialog = new FTPWindowImpl(tr("Uploading Profile: "), Profile->text());
		dialog->show();
		dialog->Upload(Profiles[i], Profiles[i]->Files);
	}
}

void MainWindowImpl::StartDownload(QAction *Profile) {
	if (CheckForPassword(0)) {
		int i = 0;
		while (Profiles[i]->Name != Profile->text()) i++;
		LW_Profiles->setCurrentRow(i);
		UpdateFileStatusButton();
		FTPWindowImpl *dialog;
		if (Settings->value("Proxy/UseProxy", 0).toBool()) dialog = new FTPWindowImpl(tr("Downloading: "), Profile->text(), Settings->value("Proxy/ProxyIP", "127.0.0.1").toString(), Settings->value("Proxy/ProxyPort", "80").toString()); else
			dialog = new FTPWindowImpl(tr("Downloading Profile: "), Profile->text());
		dialog->show();
		dialog->Download(Profiles[i], Profiles[i]->Files);
	}
}

void MainWindowImpl::StartUploadButton() {
	if (CheckForPassword(0)) {
		int i = LW_Profiles->currentRow();
			if (i >= 0) {
			if (UpdateFileStatusButton()) {
				FTPWindowImpl *dialog;
				if (Settings->value("Proxy/UseProxy", 0).toBool()) dialog = new FTPWindowImpl(tr("Uploading: "), Profiles[i]->Name, Settings->value("Proxy/ProxyIP", "127.0.0.1").toString(), Settings->value("Proxy/ProxyPort", "80").toString()); else
					dialog = new FTPWindowImpl(tr("Uploading Profile: "), Profiles[i]->Name);
				dialog->show();
				dialog->Upload(Profiles[i], Profiles[i]->Files);
			}
		}
	}
}

void MainWindowImpl::StartDownloadButton() {
	if (CheckForPassword(0)) {
		int i = LW_Profiles->currentRow();
		if (i >= 0) {
			if (UpdateFileStatusButton()) {
				FTPWindowImpl *dialog;
				if (Settings->value("Proxy/UseProxy", 0).toBool()) dialog = new FTPWindowImpl(tr("Downloading: "), Profiles[i]->Name, Settings->value("Proxy/ProxyIP", "0").toString(), Settings->value("Proxy/ProxyPort", "80").toString()); else
					dialog = new FTPWindowImpl(tr("Downloading Profile: "), Profiles[i]->Name);
				dialog->show();
				dialog->Download(Profiles[i], Profiles[i]->Files);
			}
		}
	}
}

void MainWindowImpl::RemoveProfileButton() {
	std::vector<OProfile*>::iterator i = Profiles.begin();

	for (int j = 0; j < LW_Profiles->currentRow(); j++) i++;

	Profiles.erase(i);
	SaveProfiles();
	SetupMenus();
	ShowProfiles(0);
}

void MainWindowImpl::AddFileButton() {
	QString FileLocal;
	int i = LW_Profiles->currentRow();
	FileLocal = QFileDialog::getSaveFileName(this, tr("Chose location for downloaded file"), Profiles[i]->LocalBaseDir, tr("All Files (*.*)"), 0, QFileDialog::DontConfirmOverwrite);
	if (!FileLocal.isEmpty()) {
		FileLocal.remove(0, Profiles[i]->LocalBaseDir.size());
		BrowseFTPImpl *dialog = new BrowseFTPImpl(Profiles[i], FileLocal.split("/").last());
		dialog->exec();
		if (!dialog->Result.isEmpty()) {
			dialog->Result.remove(0, Profiles[i]->ServerBaseDir.size()-1);
			Profiles[i]->Files.push_back(new OFileConnection(FileLocal, dialog->Result));
			ShowProfiles(i);
		}
		delete dialog;
	}
	SaveProfiles();
	LW_Profiles->setCurrentRow(i);
	ShowProfileDetails(i);
}

void MainWindowImpl::RemoveFileButton() {
	if ((TW_Server->selectedItems().size() == 1) && (TW_Local->selectedItems().size() == 1)) {
		int i = LW_Profiles->currentRow();
		QTreeWidgetItem *LocalItem = TW_Local->currentItem();
		QTreeWidgetItem *ServerItem = TW_Server->currentItem();
		
		if (!Profiles[i]->DeleteConnection(LocalItem, ServerItem)) QMessageBox::critical(this, tr("Error"), tr("Could not be deleted (BUG)"));;;
		
		SaveProfiles();
		ShowProfileDetails(i);
	}
	else QMessageBox::information(this, tr("Selected connection not unique"), tr("Select a unique connection (one file connected to exactly one file)."));
}

void MainWindowImpl::RenameLocalFile() {
	QString FileLocal;
	int i = LW_Profiles->currentRow();
	FileLocal = QFileDialog::getSaveFileName(this, tr("Chose location for downloaded file"), Profiles[i]->LocalBaseDir, tr("All Files (*.*)"), 0, QFileDialog::DontConfirmOverwrite);
	if (!FileLocal.isEmpty()) {
		QTreeWidgetItem *LocalItem = TW_Local->currentItem();
		
		OFileConnection *C  = Profiles[i]->LookForLocalWidgetItem(LocalItem)[0];
		FileLocal.remove(0, Profiles[i]->LocalBaseDir.size());
		C->LocalFile = FileLocal;

		SaveProfiles();
		ShowProfileDetails(i);
	}
}

void MainWindowImpl::RenameServerFile() {
	QString FileServer;
	int i = LW_Profiles->currentRow();

	QTreeWidgetItem *ServerItem = TW_Server->currentItem();
	OFileConnection *C  = Profiles[i]->LookForServerWidgetItem(ServerItem)[0];

	BrowseFTPImpl *dialog = new BrowseFTPImpl(Profiles[i], C->LocalFile.split("/").last());
	dialog->exec();
	if (!dialog->Result.isEmpty()) {
		dialog->Result.remove(0, Profiles[i]->ServerBaseDir.size()-1);
		C->ServerFile = dialog->Result;

		SaveProfiles();
		ShowProfileDetails(i);
	}
}

void MainWindowImpl::ShowInfoBox() {
	if ((TW_Server->selectedItems().size() == 1) && (TW_Local->selectedItems().size() == 1)) {
		int i = LW_Profiles->currentRow();
		QTreeWidgetItem *LocalItem = TW_Local->currentItem();
		QTreeWidgetItem *ServerItem = TW_Server->currentItem();
		
		OFileConnection *FileConnection = Profiles[i]->LookForWidgetItems(LocalItem, ServerItem);
		if (FileConnection != NULL) {
			std::vector<OFileConnection*> Result;
			Result.push_back(FileConnection);
			UpdateFileStatusDialogImpl *dialog1 = new UpdateFileStatusDialogImpl(Profiles[i], Settings->value("Proxy/ProxyIP", "0").toString(), Settings->value("Proxy/ProxyPort", "80").toString(), Result);
			dialog1->exec();
			FileInfoDialogImpl *dialog2 = new FileInfoDialogImpl(FileConnection, Profiles[i]);
			dialog2->exec();
		}
	}
	else QMessageBox::information(this, tr("Selected connection not unique"), tr("Select a unique connection (one file connected to exactly one file)."));
}

bool MainWindowImpl::UpdateFileStatusButton() {
	int i = LW_Profiles->currentRow();
	if (i>=0) {
		UpdateFileStatusDialogImpl *dialog = new UpdateFileStatusDialogImpl(Profiles[i], Settings->value("Proxy/ProxyIP", "0").toString(), Settings->value("Proxy/ProxyPort", "80").toString());
		dialog->exec();
		//QMessageBox::information(this, "Debug", "TimeDiff: " + QString::number(Profiles[i]->TimeDifference));
		/*for (std::vector<OFileConnection*>::iterator j= Profiles[i]->Files.begin(); j < Profiles[i]->Files.end(); j++) {
			QMessageBox::information(this, "Datei", (*j)->LocalFile + " - " +(*j)->ServerFile + "\n" +
								(*j)->LastModifiedLocal + " - " +(*j)->LastModifiedServer + "\n" +
								QString::number((*j)->SizeLocal) + " - " + QString::number((*j)->SizeServer));
		}*/
		
		
		if (!dialog->Accepted) return 0;
		delete dialog;
		ShowProfileDetails(i);
	}
	return 1;
}

void MainWindowImpl::SaveProfiles() {
	QDir *ConfDir = new QDir(QDir::homePath()+"/.oneclickftp");
	if (!(ConfDir->exists())) ConfDir->mkdir(QDir::homePath()+"/.oneclickftp");

	delete ConfDir;
		
	QBuffer Buffer;
	Buffer.open(QIODevice::ReadWrite);
	QTextStream OutputStream(&Buffer);
	
	OutputStream << ("0.5.0\n");
	OutputStream << (Profiles.size());
	OutputStream << ("\n");
	for (std::vector<OProfile*>::iterator i = Profiles.begin(); i < Profiles.end();i++) {
		OutputStream << (*i)->Name+"\n";
		OutputStream << (*i)->Files.size();
		OutputStream << ("\n");
		for (std::vector<OFileConnection*>::iterator j = (*i)->Files.begin(); j < (*i)->Files.end();j++) OutputStream << (*j)->LocalFile+"\n";
		for (std::vector<OFileConnection*>::iterator j = (*i)->Files.begin(); j < (*i)->Files.end();j++) OutputStream << (*j)->ServerFile+"\n";
		OutputStream << (*i)->Server+"\n";
		OutputStream <<(*i)->Username+"\n";
		OutputStream << (*i)->Password+"\n";
		OutputStream << (*i)->LocalBaseDir+"\n";
		OutputStream << (*i)->ServerBaseDir+"\n";
		if ((*i)->CheckForMDTM) OutputStream << "True\n"; else OutputStream << "False\n";
		if ((*i)->CheckForSize) OutputStream << "True\n"; else OutputStream << "False\n";
	}
	
	QByteArray SecureArray;
	QByteArray BufferArray;
	Buffer.close();
	
	BufferArray = Buffer.data();
	
	QCA::Initializer Init;
	QCA::SymmetricKey Key(QByteArray("Thisisthefuckingpassword"));
	QCA::InitializationVector InitVec(QByteArray("Thisisthefuckingpassword"));
	QCA::Cipher Cipher(QString("blowfish"), QCA::Cipher::CBC, QCA::Cipher::DefaultPadding, QCA::Encode, Key, InitVec);
	QCA::SecureArray EncryptedArray = Cipher.process(QCA::SecureArray(BufferArray));
	
	QFile ConfFile(QDir::homePath()+"/.oneclickftp/profiles");
	ConfFile.open(QFile::ReadWrite);
	ConfFile.write(EncryptedArray.toByteArray());
	ConfFile.close();
}

void MainWindowImpl::SaveProfilesDebug() {
	QDir *ConfDir = new QDir(QDir::homePath()+"/.oneclickftp");
	if (!(ConfDir->exists())) ConfDir->mkdir(QDir::homePath()+"/.oneclickftp");
	delete ConfDir;
	
	QFile ConfFile(QDir::homePath()+"/.oneclickftp/profilesdebug");
	ConfFile.open(QFile::ReadWrite);
	
	QTextStream OutputStream(&ConfFile);
	
	OutputStream << ("0.5.0\n");
	OutputStream << (Profiles.size());
	OutputStream << ("\n");
	for (std::vector<OProfile*>::iterator i = Profiles.begin(); i < Profiles.end();i++) {
		OutputStream << (*i)->Name+"\n";
		OutputStream << (*i)->Files.size();
		OutputStream << ("\n");
		for (std::vector<OFileConnection*>::iterator j = (*i)->Files.begin(); j < (*i)->Files.end();j++) OutputStream << (*j)->LocalFile+"\n";
		for (std::vector<OFileConnection*>::iterator j = (*i)->Files.begin(); j < (*i)->Files.end();j++) OutputStream << (*j)->ServerFile+"\n";
		OutputStream << (*i)->Server+"\n";
		OutputStream <<(*i)->Username+"\n";
		OutputStream << (*i)->Password+"\n";
		OutputStream << (*i)->LocalBaseDir+"\n";
		OutputStream << (*i)->ServerBaseDir+"\n";
		if ((*i)->CheckForMDTM == 1) OutputStream << "True\n"; else OutputStream << "False\n";
		if (((*i)->CheckForSize)) OutputStream << "True\n"; else OutputStream << "False\n";
	}

	ConfFile.close();
}

void MainWindowImpl::AddProfileButton() {
	AddProfileImpl *dialog = new AddProfileImpl(this);
	dialog->exec();
	if (dialog->Accepted) {
		Profiles.push_back(dialog->NewProfile);
		SaveProfiles();
		SetupMenus();
		ShowProfiles(0);
	}
	delete dialog;
}

void MainWindowImpl::LWProfilesRowChanged(int NewRow) {
	if (Changed) {
		QMessageBox MsgBox(QMessageBox::Question, tr("Profile canged"), tr("The selected Profile has been changed. Do you want to save these changes?"));
		MsgBox.addButton(QMessageBox::Yes);
		MsgBox.addButton(QMessageBox::No);
		if (MsgBox.exec() == QMessageBox::No) {
			Profiles[ProfileChanged]->Server = OldProfile->Server;
			Profiles[ProfileChanged]->Username = OldProfile->Username;
			Profiles[ProfileChanged]->Password = OldProfile->Password;
			Profiles[ProfileChanged]->CheckForMDTM = OldProfile->CheckForMDTM;
			Profiles[ProfileChanged]->CheckForSize = OldProfile->CheckForSize;
		}
	}
	SaveProfiles();
	if (NewRow >= 0) ShowProfileDetails(NewRow);
	Changed = 0;
}

void MainWindowImpl::ProfileDetailsChanged() {
	ProfileChanged = LW_Profiles->currentRow();
	Changed = 1;
}

void MainWindowImpl::LE_Changed(QString) {
	int i = LW_Profiles->currentRow();
	if (i>=0) {
		ProfileDetailsChanged();
		Profiles[i]->Server = LE_Servername->text();
		Profiles[i]->Username = LE_Username->text();
		Profiles[i]->Password = LE_Password->text();
		Profiles[i]->ServerBaseDir = LE_BaseDir_Server->text();
		Profiles[i]->LocalBaseDir = LE_BaseDir_Local->text();
	}
}

void MainWindowImpl::CB_CheckForMDTM_Changed(int ChangedTo) {
	int i = LW_Profiles->currentRow();
	if (i >= 0) {
		ProfileDetailsChanged();
		if (ChangedTo == 2) {
			Profiles[i]->CheckForMDTM = 1;
			Profiles[i]->CheckForSize = 0;
			CB_CheckForSize->setCheckState(Qt::Unchecked);
		} else
		if (ChangedTo == 0)
			Profiles[i]->CheckForMDTM = 0;
		}
	
}

void MainWindowImpl::CB_CheckForSize_Changed(int ChangedTo) {
	int i = LW_Profiles->currentRow();
	if (i >= 0) {
		ProfileDetailsChanged();
		if (ChangedTo == 2) {
			Profiles[i]->CheckForMDTM = 0;
			Profiles[i]->CheckForSize = 1;
			CB_CheckForMDTM->setCheckState(Qt::Unchecked);
		} else
		if (ChangedTo == 0)
			Profiles[i]->CheckForSize = 0;
	}
}

void MainWindowImpl::ExitApp() {
	Settings->setValue("Geometry", this->saveGeometry());

	LWProfilesRowChanged(-1);

	delete OldProfile;

	for (std::vector<OProfile*>::iterator itP = Profiles.begin(); itP<Profiles.end(); itP++) {
		for (std::vector<OFileConnection*>::iterator itF = (*itP)->Files.begin(); itF < (*itP)->Files.end(); itF++)
			if ((*itF) != NULL) delete (*itF);
		if ((*itP) != NULL) delete (*itP);
	}

	QCoreApplication::exit();
	}

void MainWindowImpl::AboutApp() {
	QMessageBox::about(this, tr("About OneClickFTP"), tr("<center><span style=\"margin-left:50px; font-weight:bold; font-size:24pt;\">OneClickFTP</span><br><br><br>Author: Stefan Lau<br>")+ Translator +tr("<br>Homepage: <a href=http://sourceforge.net/projects/oneclickftp><span style=\"text-decoration: underline; color:#0000ff;\">http://sourceforge.net/projects/oneclickftp</span></a><br><br>Version: 0.2 CVS</center>"));
}

void MainWindowImpl::SettingsButton() {
	SettingsImpl *dialog = new SettingsImpl(this, 0, Settings);
	dialog->exec();
	delete dialog;
}

void MainWindowImpl::BaseDirLocalSeekButton() {
	int i = LW_Profiles->currentRow();
	QString BaseDir = QFileDialog::getExistingDirectory(this, tr("Chose local base-directory"), Home.path());
	if (!BaseDir.isEmpty()) {
		Profiles[i]->LocalBaseDir = BaseDir;
		SaveProfiles();
		ShowProfiles(i);
	}
}

void MainWindowImpl::BaseDirServerSeekButton() {
	int i = LW_Profiles->currentRow();
	BrowseFTPImpl *dialog = new BrowseFTPImpl(Profiles[i], "", 1);
	dialog->exec();
	if (!dialog->Result.isEmpty()) {
		Profiles[i]->ServerBaseDir = dialog->Result;
		SaveProfiles();
		ShowProfiles(i);	
	}
	delete dialog;
}

void MainWindowImpl::RenameProfileButton() {
	int j = LW_Profiles->currentRow();
	QString NewName = QInputDialog::getText(this, tr("Enter new name"), tr("Enter the new name for the profile")+QString(" \"")+Profiles[j]->Name+QString("\""));
	if (!NewName.isEmpty()) {
		bool Exists = 0;
			
		for (std::vector<OProfile*>::iterator i = Profiles.begin(); i < Profiles.end(); i++)
			if ((*i)->Name == NewName) Exists = 1;
		if (NewName == Profiles[j]->Name) Exists = 0;

		if (!Exists) Profiles[j]->Name = NewName;
		else QMessageBox::information(this, tr("Error"), tr("This name already exists. Please enter another name."));
	}
	SetupMenus();
	ShowProfiles(j);
}

void MainWindowImpl::ExportProfilesButton() {
	ExportProfilesImpl *dialog = new ExportProfilesImpl(this);
	dialog->exec();
	if (dialog->Accepted) {
		QString Password = QInputDialog::getText(this, tr("Password"), tr("Enter a password to protect your information."), QLineEdit::Password);
		if (!Password.isEmpty()) {
			QDir *ConfDir = new QDir(QDir::homePath()+"/.oneclickftp");
			if (!(ConfDir->exists())) ConfDir->mkdir(QDir::homePath()+"/.oneclickftp");
			delete ConfDir;
		
			QBuffer Buffer;
			Buffer.open(QIODevice::ReadWrite);
			QTextStream OutputStream(&Buffer);
	
			OutputStream << ("0.5.0\n");
			OutputStream << (dialog->Result.size());
			OutputStream << ("\n");
			for (std::vector<OProfile*>::iterator i = dialog->Result.begin(); i < dialog->Result.end();i++) {
				OutputStream << (*i)->Name+"\n";
				OutputStream << (*i)->Files.size();
				OutputStream << ("\n");
				for (std::vector<OFileConnection*>::iterator k = (*i)->Files.begin(); k < (*i)->Files.end();k++) OutputStream << (*k)->LocalFile+"\n";
				for (std::vector<OFileConnection*>::iterator k = (*i)->Files.begin(); k < (*i)->Files.end();k++) OutputStream << (*k)->ServerFile+"\n";
				OutputStream << (*i)->Server+"\n";
				OutputStream << (*i)->Username+"\n";
				OutputStream << (*i)->Password+"\n";
				OutputStream << (*i)->LocalBaseDir+"\n";
				OutputStream << (*i)->ServerBaseDir+"\n";
				if ((*i)->CheckForMDTM == 1) OutputStream << "True\n"; else OutputStream << "False\n";
				if (((*i)->CheckForSize)) OutputStream << "True\n"; else OutputStream << "False\n";
			}
	
			QByteArray SecureArray;
			QByteArray BufferArray;
			Buffer.close();
	
			BufferArray = Buffer.data();
	
			QCA::Initializer Init;
			QCA::SymmetricKey Key(Password.toUtf8());
			QCA::InitializationVector InitVec(QByteArray("InitializationVector"));
			QCA::Cipher Cipher(QString("blowfish"), QCA::Cipher::CBC, QCA::Cipher::DefaultPadding, QCA::Encode, Key, InitVec);
			QCA::SecureArray EncryptedArray = Cipher.process(QCA::SecureArray(BufferArray));
	
			QString SaveFile = QFileDialog::getSaveFileName(this, tr("Save exported profiles at ..."), QDir::homePath());
			if (!SaveFile.isEmpty()) {
				QFile ConfFile(SaveFile);
				ConfFile.open(QFile::ReadWrite);
				ConfFile.write(EncryptedArray.toByteArray());
				ConfFile.close();
			}
		}
	}
	delete dialog;
}

void MainWindowImpl::ImportProfilesButton() {
	QString LoadFile = QFileDialog::getOpenFileName(this, tr("Load exported profiles from ..."), QDir::homePath());
	if (!LoadFile.isEmpty()) {
		QString Password = QInputDialog::getText(this, tr("Password"), tr("Enter the Password for this Data."), QLineEdit::Password);
		if (!Password.isEmpty()) {
			QFile *ConfFile = new QFile(LoadFile);
			ConfFile->open(QFile::ReadWrite);
			
			QCA::Initializer Init;
			QCA::SecureArray EncryptedArray = ConfFile->readAll();
			ConfFile->close();
			delete ConfFile;
		
			QCA::SymmetricKey Key(QByteArray(Password.toUtf8()));
			QCA::InitializationVector InitVec(QByteArray("InitializationVector"));
			QCA::Cipher Cipher(QString("blowfish"), QCA::Cipher::CBC, QCA::Cipher::DefaultPadding, QCA::Decode, Key, InitVec);	
			QBuffer Buffer;
			Buffer.setData(Cipher.process(EncryptedArray).toByteArray());	
			Buffer.open(QIODevice::ReadWrite);
			QTextStream *InputStream = new QTextStream(&Buffer);
		
			QString Version = InputStream->readLine();
			if (Version == "0.3.0") Import03(InputStream, this); else
			if (Version == "0.5.0") Import03(InputStream, this);
			else QMessageBox::information(this, tr("Error"), tr("The File does not contain usable data. (Maybe the password was wrong)"));
			ShowProfiles(0);
			SetupMenus();
			SaveProfiles();
		}
	}
}

bool MainWindowImpl::Import03(QTextStream *InputStream, MainWindowImpl *win) {
	QString Convert = InputStream->readLine();
	for (int i=0; i < Convert.toInt();i++) {
		bool Jump = 0;

		OProfile *NewProfile = new OProfile();
		NewProfile->Name = InputStream->readLine();
		for (std::vector<OProfile*>::iterator it = win->Profiles.begin(); it < win->Profiles.end(); it++)
			if ((*it)->Name == NewProfile->Name) {
				QString NewName = QInputDialog::getText(this, tr("Enter new name"), tr("A Profile with the name \"") + NewProfile->Name + tr("\" already exists, please choose another name or cancel the import of this profile"));
				if (NewName.size() > 0) {
					NewProfile->Name = NewName;
				} else Jump = 1;
			}
			
		QString Convert2 = InputStream->readLine();
			
		for (int j=0; j < Convert2.toInt();j++)
			NewProfile->Files.push_back(new OFileConnection);

		for (int j=0; j < Convert2.toInt();j++) {
			QString temp = InputStream->readLine();
			NewProfile->Files[j]->LocalFile = temp;
			}	
			
		for (int j=0; j < Convert2.toInt();j++) {
			QString temp = InputStream->readLine();			
			NewProfile->Files[j]->ServerFile = temp;
			
		}		
			
		NewProfile->Server = InputStream->readLine();
		NewProfile->Username = InputStream->readLine();
		NewProfile->Password = InputStream->readLine();
		NewProfile->LocalBaseDir = InputStream->readLine();
		NewProfile->ServerBaseDir = InputStream->readLine();

		if (!Jump) win->Profiles.push_back(NewProfile); else delete NewProfile;
		Jump = 0;
	}
	return 1;
}


bool MainWindowImpl::Import05(QTextStream *InputStream, MainWindowImpl *win) {
	QString Convert = InputStream->readLine();
	for (int i=0; i < Convert.toInt();i++) {
		bool Jump = 0;

		OProfile *NewProfile = new OProfile();
		NewProfile->Name = InputStream->readLine();
		for (std::vector<OProfile*>::iterator it = win->Profiles.begin(); it < win->Profiles.end(); it++)
			if ((*it)->Name == NewProfile->Name) {
				QString NewName = QInputDialog::getText(this, tr("Enter new name"), tr("A Profile with the name \"") + NewProfile->Name + tr("\" already exists, please choose another name or cancel the import of this profile"));
				if (NewName.size() > 0) {
					NewProfile->Name = NewName;
				} else Jump = 1;
			}
			
		QString Convert2 = InputStream->readLine();
			
		for (int j=0; j < Convert2.toInt();j++)
			NewProfile->Files.push_back(new OFileConnection);

		for (int j=0; j < Convert2.toInt();j++) {
			QString temp = InputStream->readLine();
			NewProfile->Files[j]->LocalFile = temp;
			}	
			
		for (int j=0; j < Convert2.toInt();j++) {
			QString temp = InputStream->readLine();			
			NewProfile->Files[j]->ServerFile = temp;
			
		}		
			
		NewProfile->Server = InputStream->readLine();
		NewProfile->Username = InputStream->readLine();
		NewProfile->Password = InputStream->readLine();
		NewProfile->LocalBaseDir = InputStream->readLine();
		NewProfile->ServerBaseDir = InputStream->readLine();
		if (InputStream->readLine() == "True") NewProfile->CheckForMDTM = 1; else NewProfile->CheckForMDTM = 0;
		if (InputStream->readLine() == "True") NewProfile->CheckForSize = 1; else NewProfile->CheckForSize = 0;

		if (!Jump) win->Profiles.push_back(NewProfile); else delete NewProfile;
		Jump = 0;
	}
	return 1;
}

